
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: db.c,v 1.14 2003/01/03 20:01:37 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */
/* This module was created to share connections between modules 
   so that the database wasn't being opened multiple times over the 
   course of execution.
   Later it might be a good idea to encapsulate all db functionality 
   in here with a do_query, get_results, etc type functions so we can
   port to other databases more easily.
*/

#include <ctype.h>

/* Postgres includes */
#include <postgresql/libpq-fe.h>

/* Project includes */
#include "db.h"
#include "magic-smtpd.h"
#include "config_msd.h"
#include "config_qmail.h"
#include "utils.h"

PGresult *doquery(char *query);

/* Global connection */

PGconn *db;
static PGresult *res = NULL;

/* Open a connection to the database for the program to use
   so we don't open and close it a bunch of times */
int db_init(void)
{
  int retval = 0;
  char buffer[1024];

  /* If the db was already opened, don't do anything */
  if(db != NULL)
    return 1;
	
  /* Build the connect options for the database */
  snprintf(buffer, sizeof(buffer),
	   "dbname=%s host=%s port=%s user=%s password=%s",
	   LM_STRING_BUFFER(config.dbname), LM_STRING_BUFFER(config.dbhost), 
	   LM_STRING_BUFFER(config.dbport), LM_STRING_BUFFER(config.dbuser), 
	   LM_STRING_BUFFER(config.dbpwd));

  /* make a connection to the database */
  db = PQconnectdb(buffer);
  if(db == NULL || PQstatus(db) == CONNECTION_BAD)
    {
      /* Print out some error messages */
      LM_LOG_WARN(MSD_LOG_SYS, "Could not connect to database.");
      LM_LOG_ERROR(MSD_LOG_SYS, PQerrorMessage(db));
      SYSLOG((LOG_ERR, "Connection to database %s (host %s, port %s, user %s) failed.", 
	      LM_STRING_BUFFER(config.dbname), LM_STRING_BUFFER(config.dbhost), 
	      LM_STRING_BUFFER(config.dbport), LM_STRING_BUFFER(config.dbuser)));
      SYSLOG((LOG_ERR, PQerrorMessage(db)));
      db = NULL;
      retval = -1;
    }

  return retval;
}

int db_cleanup(void)
{
  int retval = 0;

  if(res != NULL)
    {
      PQclear(res);
      res = NULL;
    }
  
  PQfinish(db);
  db = NULL;
  return retval;
}

/* seperate a "user@domain" string into two escaped strings for creating queries. 
   if @domain isn't found in the string, use the default domain */
int msd_db_get_user_domain(char **pescuser, char **pescdomain, lm_string_t useratdomain)
{
  lm_string_t username = NULL;
  lm_string_t domain = NULL;
  int index;
  char *escuser = NULL;
  char *escdomain = NULL;
  char *buffer;

  if(get_user_domain(&username, &domain, useratdomain))
    msd_error_out_of_memory();
  
  if(LM_STRING_LEN(domain) == 0)
    if(LM_STRING_CAT(domain, config_qmail.defaultdomain))
      msd_error_out_of_memory();

  /* HACK if sent to localhost address use qmail's default 
     domain to valid user checking */
  buffer = LM_STRING_BUFFER(domain);
  if(tolower(buffer[0]) == 'l' &&
     tolower(buffer[1]) == 'o' &&
     tolower(buffer[2]) == 'c' &&
     tolower(buffer[3]) == 'a' &&
     tolower(buffer[4]) == 'l' &&
     tolower(buffer[5]) == 'h' &&
     tolower(buffer[6]) == 'o' &&
     tolower(buffer[7]) == 's' &&
     tolower(buffer[8]) == 't' &&
     buffer[9] == '\0')
    {
      LM_STRING_FREE(domain);
      if(LM_STRING_DUP(domain, config_qmail.defaultdomain))
	msd_error_out_of_memory();
    }
     
  index = (LM_STRING_LEN(username)*2)+1;
  if(LM_MALLOC(escuser, index))
    msd_error_out_of_memory();
  index = (LM_STRING_LEN(domain)*2)+1;
  if(LM_MALLOC(escdomain, index))
    msd_error_out_of_memory();
  PQescapeString(escuser, LM_STRING_BUFFER(username), LM_STRING_LEN(username));
  PQescapeString(escdomain, LM_STRING_BUFFER(domain), LM_STRING_LEN(domain));

  LM_STRING_FREE(username);
  LM_STRING_FREE(domain);
  *pescuser = escuser;
  *pescdomain = escdomain;
  return 0;
}

/* Perform a query 
   returns 0 if the query was ok and records were returned
   returns 1 if the query was ok but no records were returned
   returns -1 if there was an error with the query
*/
int db_do_query(PGresult **presult, char *query)
{
  PGresult *result;
  int retval = -1;

  result = doquery(query);
  
  if(PQresultStatus(result) == PGRES_TUPLES_OK)
    {
      if(PQntuples(result) > 0)
	{
	  /* records returned */
	  retval = 0;
	}
      else
	{
	  /* no records */
	  retval = 1;
	}
    }
  else
    {
      /* error with executing query */
      retval = -1;
    }

  /* give back the result struct if the user wants it */
  if(presult)
    *presult = result;
  
  return retval;
}

PGresult *doquery(char *query)
{
  LM_WARN(db == NULL);
  
  if(res != NULL)
    PQclear(res);
  
  res = PQexec(db, query);
  
  if ((res == NULL ||
       PQresultStatus(res) == PGRES_BAD_RESPONSE ||
       PQresultStatus(res) == PGRES_NONFATAL_ERROR ||
       PQresultStatus(res) == PGRES_FATAL_ERROR))
    {
      if (res != NULL)
	fprintf(stderr, "query error:  %s\n", PQcmdStatus(res));
      else
	fprintf(stderr, "connection error:  %s\n", PQerrorMessage(db));
      PQfinish(db);
    }
  return res;
}

